@@ -24,6 +24,7 @@ from samba.ndr import ndr_unpack, ndr_pack
 from samba.dcerpc import dnsp
 from samba.tests.samba_tool.base import SambaToolCmdTest
 
+
 class DnsCmdTestCase(SambaToolCmdTest):
     def setUp(self):
         super(DnsCmdTestCase, self).setUp()
@@ -44,9 +45,8 @@ class DnsCmdTestCase(SambaToolCmdTest):
 
         good_dns = ["SAMDOM.EXAMPLE.COM",
                     "1.EXAMPLE.COM",
-                    "%sEXAMPLE.COM" % ("1."*100),
+                    "%sEXAMPLE.COM" % ("1." * 100),
                     "EXAMPLE",
-                    "\n.COM",
                     "!@#$%^&*()_",
                     "HIGH\xFFBYTE",
                     "@.EXAMPLE.COM",
@@ -57,17 +57,19 @@ class DnsCmdTestCase(SambaToolCmdTest):
                    "",
                    "SAMDOM..EXAMPLE.COM"]
 
-        good_mx = ["SAMDOM.EXAMPLE.COM 65530"]
+        good_mx = ["SAMDOM.EXAMPLE.COM 65530",
+                   "SAMDOM.EXAMPLE.COM     0"]
         bad_mx = ["SAMDOM.EXAMPLE.COM -1",
                   "SAMDOM.EXAMPLE.COM",
                   " ",
                   "SAMDOM.EXAMPLE.COM 1 1",
                   "SAMDOM.EXAMPLE.COM SAMDOM.EXAMPLE.COM"]
 
-        good_srv = ["SAMDOM.EXAMPLE.COM 65530 65530 65530"]
+        good_srv = ["SAMDOM.EXAMPLE.COM 65530 65530 65530",
+                    "SAMDOM.EXAMPLE.COM     1     1     1"]
         bad_srv = ["SAMDOM.EXAMPLE.COM 0 65536 0",
                    "SAMDOM.EXAMPLE.COM 0 0 65536",
-                   "SAMDOM.EXAMPLE.COM 65536 0 0" ]
+                   "SAMDOM.EXAMPLE.COM 65536 0 0"]
 
         for bad_dn in bad_dns:
             bad_mx.append("%s 1" % bad_dn)
@@ -88,12 +90,12 @@ class DnsCmdTestCase(SambaToolCmdTest):
                         "::1",
                         "::",
                         "1:1:1:1:1:1:1:1"],
-                "PTR":good_dns,
-                "CNAME":good_dns,
-                "NS":good_dns,
-                "MX":good_mx,
-                "SRV":good_srv,
-                "TXT":["text", "", "@#!", "\n"]
+                "PTR": good_dns,
+                "CNAME": good_dns,
+                "NS": good_dns,
+                "MX": good_mx,
+                "SRV": good_srv,
+                "TXT": ["text", "", "@#!", "\n"]
         }
 
         self.bad_records = {
@@ -105,11 +107,11 @@ class DnsCmdTestCase(SambaToolCmdTest):
                         "1234:5678:9ABC:DEF0:1234:5678:9ABC:DEF0:1234",
                         "1234:5678:9ABC:DEF0:1234:5678:9ABC",
                         "1111::1111::1111"],
-                "PTR":bad_dns,
-                "CNAME":bad_dns,
-                "NS":bad_dns,
-                "MX":bad_mx,
-                "SRV":bad_srv
+                "PTR": bad_dns,
+                "CNAME": bad_dns,
+                "NS": bad_dns,
+                "MX": bad_mx,
+                "SRV": bad_srv
         }
 
     def tearDown(self):
@@ -171,7 +173,7 @@ class DnsCmdTestCase(SambaToolCmdTest):
                               % (record_str, record_type_str))
 
         dn, record = self.get_record_from_db(self.zone, "testrecord")
-        record.rank = 0 # DNS_RANK_NONE
+        record.rank = 0  # DNS_RANK_NONE
         res = self.samdb.dns_replace_by_dn(dn, [record])
         if res is not None:
             self.fail("Unable to update dns record to have DNS_RANK_NONE.")
@@ -184,12 +186,12 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                           self.creds_string)
         try:
             self.assertCmdSuccess(result, out, err,
-                                  "Failed to query for a record" \
+                                  "Failed to query for a record"
                                   "which had DNS_RANK_NONE.")
             self.assertTrue("testrecord" in out and record_str in out,
-                            "Query for a record which had DNS_RANK_NONE" \
+                            "Query for a record which had DNS_RANK_NONE"
                             "succeeded but produced no resulting records.")
-        except AssertionError, e:
+        except AssertionError as e:
             # Windows produces no resulting records
             pass
 
@@ -198,9 +200,9 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                           self.zone, "testrecord", record_type_str,
                                           record_str, self.creds_string)
         try:
-            self.assertCmdFail(result, "Successfully added duplicate record" \
+            self.assertCmdFail(result, "Successfully added duplicate record"
                                "of one which had DNS_RANK_NONE.")
-        except AssertionError, e:
+        except AssertionError as e:
             errors.append(e)
 
         # We should be able to delete it
@@ -208,9 +210,9 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                           self.zone, "testrecord", record_type_str,
                                           record_str, self.creds_string)
         try:
-            self.assertCmdSuccess(result, out, err, "Failed to delete record" \
+            self.assertCmdSuccess(result, out, err, "Failed to delete record"
                                   "which had DNS_RANK_NONE.")
-        except AssertionError, e:
+        except AssertionError as e:
             errors.append(e)
 
         # Now the record should not exist
@@ -218,9 +220,9 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                           self.zone, "testrecord",
                                           record_type_str, self.creds_string)
         try:
-            self.assertCmdFail(result, "Successfully queried for deleted record" \
+            self.assertCmdFail(result, "Successfully queried for deleted record"
                                "which had DNS_RANK_NONE.")
-        except AssertionError, e:
+        except AssertionError as e:
             errors.append(e)
 
         if len(errors) > 0:
@@ -243,7 +245,7 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                                       self.zone, "testrecord",
                                                       dnstype, record,
                                                       self.creds_string)
-                    self.assertCmdSuccess(result, out, err, "Failed to add" \
+                    self.assertCmdSuccess(result, out, err, "Failed to add"
                                           "record %s with type %s."
                                           % (record, dnstype))
 
@@ -252,7 +254,7 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                                       self.zone, "testrecord",
                                                       dnstype,
                                                       self.creds_string)
-                    self.assertCmdSuccess(result, out, err, "Failed to query" \
+                    self.assertCmdSuccess(result, out, err, "Failed to query"
                                           "record %s with qualifier %s."
                                           % (record, dnstype))
 
@@ -261,7 +263,7 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                                       self.zone, "testrecord",
                                                       dnstype, record,
                                                       self.creds_string)
-                    self.assertCmdSuccess(result, out, err, "Failed to remove" \
+                    self.assertCmdSuccess(result, out, err, "Failed to remove"
                                           "record %s with type %s."
                                           % (record, dnstype))
                 except AssertionError as e:
@@ -271,7 +273,7 @@ class DnsCmdTestCase(SambaToolCmdTest):
         if num_failures > 0:
             for msg in failure_msgs:
                 print(msg)
-            self.fail("Failed to accept valid commands. %d total failures." \
+            self.fail("Failed to accept valid commands. %d total failures."
                       "Errors above." % num_failures)
 
     def test_reject_invalid_commands(self):
@@ -291,7 +293,7 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                                       self.zone, "testrecord",
                                                       dnstype, record,
                                                       self.creds_string)
-                    self.assertCmdFail(result, "Successfully added invalid" \
+                    self.assertCmdFail(result, "Successfully added invalid"
                                        "record '%s' of type '%s'."
                                        % (record, dnstype))
                 except AssertionError as e:
@@ -304,8 +306,8 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                                       self.zone, "testrecord",
                                                       dnstype, record,
                                                       self.creds_string)
-                    self.assertCmdFail(result, "Successfully deleted invalid" \
-                                       "record '%s' of type '%s' which" \
+                    self.assertCmdFail(result, "Successfully deleted invalid"
+                                       "record '%s' of type '%s' which"
                                        "shouldn't exist." % (record, dnstype))
                 except AssertionError as e:
                     num_failures = num_failures + 1
@@ -324,7 +326,7 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                                       self.zone, "testrecord",
                                                       dnstype, good_record,
                                                       self.creds_string)
-                    self.assertCmdSuccess(result, out, err, "Failed to add " \
+                    self.assertCmdSuccess(result, out, err, "Failed to add "
                                           "record '%s' with type %s."
                                           % (record, dnstype))
 
@@ -334,8 +336,8 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                                       dnstype, good_record,
                                                       bad_record,
                                                       self.creds_string)
-                    self.assertCmdFail(result, "Successfully updated valid " \
-                                       "record '%s' of type '%s' to invalid " \
+                    self.assertCmdFail(result, "Successfully updated valid "
+                                       "record '%s' of type '%s' to invalid "
                                        "record '%s' of the same type."
                                        % (good_record, dnstype, bad_record))
 
@@ -344,7 +346,7 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                                       self.zone, "testrecord",
                                                       dnstype, good_record,
                                                       self.creds_string)
-                    self.assertCmdSuccess(result, out, err, "Could not delete " \
+                    self.assertCmdSuccess(result, out, err, "Could not delete "
                                           "valid record '%s' of type '%s'."
                                           % (good_record, dnstype))
                 except AssertionError as e:
@@ -355,7 +357,7 @@ class DnsCmdTestCase(SambaToolCmdTest):
         if num_failures > 0:
             for msg in failure_msgs:
                 print(msg)
-            self.fail("Failed to reject invalid commands. %d total failures. " \
+            self.fail("Failed to reject invalid commands. %d total failures. "
                       "Errors above." % num_failures)
 
     def test_update_invalid_type(self):
@@ -369,7 +371,7 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                               self.zone, "testrecord",
                                               dnstype1, record1,
                                               self.creds_string)
-            self.assertCmdSuccess(result, out, err, "Failed to add " \
+            self.assertCmdSuccess(result, out, err, "Failed to add "
                                   "record %s with type %s."
                                   % (record1, dnstype1))
 
@@ -386,7 +388,7 @@ class DnsCmdTestCase(SambaToolCmdTest):
                 try:
                     self.assertCmdFail(result)
                 except AssertionError:
-                    continue # Don't check this one, because record2 _is_ a valid entry of dnstype1.
+                    continue  # Don't check this one, because record2 _is_ a valid entry of dnstype1.
 
                 # Check both ways: Give the current type and try to update,
                 # and give the new type and try to update.
@@ -395,8 +397,8 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                                   self.zone, "testrecord",
                                                   dnstype1, record1,
                                                   record2, self.creds_string)
-                self.assertCmdFail(result, "Successfully updated record '%s' " \
-                                   "to '%s', even though the latter is of " \
+                self.assertCmdFail(result, "Successfully updated record '%s' "
+                                   "to '%s', even though the latter is of "
                                    "type '%s' where '%s' was expected."
                                    % (record1, record2, dnstype2, dnstype1))
 
@@ -405,12 +407,11 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                                   self.zone, "testrecord",
                                                   dnstype2, record1, record2,
                                                   self.creds_string)
-                self.assertCmdFail(result, "Successfully updated record " \
-                                   "'%s' to '%s', even though the former " \
+                self.assertCmdFail(result, "Successfully updated record "
+                                   "'%s' to '%s', even though the former "
                                    "is of type '%s' where '%s' was expected."
                                    % (record1, record2, dnstype1, dnstype2))
 
-
     def test_update_valid_type(self):
         for dnstype in self.good_records:
             for record in self.good_records[dnstype]:
@@ -419,7 +420,7 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                                   self.zone, "testrecord",
                                                   dnstype, record,
                                                   self.creds_string)
-                self.assertCmdSuccess(result, out, err, "Failed to add " \
+                self.assertCmdSuccess(result, out, err, "Failed to add "
                                       "record %s with type %s."
                                       % (record, dnstype))
 
@@ -429,7 +430,7 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                                   self.zone, "testrecord",
                                                   dnstype, record, record,
                                                   self.creds_string)
-                self.assertCmdFail(result, "Successfully updated record " \
+                self.assertCmdFail(result, "Successfully updated record "
                                    "'%s' to be exactly the same." % record)
 
                 result, out, err = self.runsubcmd("dns", "delete",
@@ -437,7 +438,7 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                                   self.zone, "testrecord",
                                                   dnstype, record,
                                                   self.creds_string)
-                self.assertCmdSuccess(result, out, err, "Could not delete " \
+                self.assertCmdSuccess(result, out, err, "Could not delete "
                                       "valid record '%s' of type '%s'."
                                       % (record, dnstype))
 
@@ -447,10 +448,10 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                               self.zone, "testrecord",
                                               "SRV", record,
                                               self.creds_string)
-            self.assertCmdSuccess(result, out, err, "Failed to add " \
+            self.assertCmdSuccess(result, out, err, "Failed to add "
                                   "record %s with type 'SRV'." % record)
 
-            split = record.split(' ')
+            split = record.split()
             new_bit = str(int(split[3]) + 1)
             new_record = '%s %s %s %s' % (split[0], split[1], split[2], new_bit)
 
@@ -459,7 +460,7 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                               self.zone, "testrecord",
                                               "SRV", record,
                                               new_record, self.creds_string)
-            self.assertCmdSuccess(result, out, err, "Failed to update record " \
+            self.assertCmdSuccess(result, out, err, "Failed to update record "
                                   "'%s' of type '%s' to '%s'."
                                   % (record, "SRV", new_record))
 
@@ -467,7 +468,7 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                               os.environ["SERVER"],
                                               self.zone, "testrecord",
                                               "SRV", self.creds_string)
-            self.assertCmdSuccess(result, out, err, "Failed to query for " \
+            self.assertCmdSuccess(result, out, err, "Failed to query for "
                                   "record '%s' of type '%s'."
                                   % (new_record, "SRV"))
 
@@ -476,7 +477,7 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                               self.zone, "testrecord",
                                               "SRV", new_record,
                                               self.creds_string)
-            self.assertCmdSuccess(result, out, err, "Could not delete " \
+            self.assertCmdSuccess(result, out, err, "Could not delete "
                                   "valid record '%s' of type '%s'."
                                   % (new_record, "SRV"))
 
@@ -484,7 +485,7 @@ class DnsCmdTestCase(SambaToolCmdTest):
         # we can't enter the wrong current value for a given record.
         for dnstype in self.good_records:
             if len(self.good_records[dnstype]) < 3:
-                continue # Not enough records of this type to do this test
+                continue  # Not enough records of this type to do this test
 
             used_record = self.good_records[dnstype][0]
             unused_record = self.good_records[dnstype][1]
@@ -495,7 +496,7 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                               self.zone, "testrecord",
                                               dnstype, used_record,
                                               self.creds_string)
-            self.assertCmdSuccess(result, out, err, "Failed to add record %s " \
+            self.assertCmdSuccess(result, out, err, "Failed to add record %s "
                                   "with type %s." % (used_record, dnstype))
 
             result, out, err = self.runsubcmd("dns", "update",
@@ -504,8 +505,8 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                               dnstype, unused_record,
                                               new_record,
                                               self.creds_string)
-            self.assertCmdFail(result, "Successfully updated record '%s' " \
-                               "from '%s' to '%s', even though the given " \
+            self.assertCmdFail(result, "Successfully updated record '%s' "
+                               "from '%s' to '%s', even though the given "
                                "source record is incorrect."
                                % (used_record, unused_record, new_record))
 
@@ -515,10 +516,10 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                           self.zone, "testrecord",
                                           "SOA", "test",
                                           self.creds_string)
-        self.assertCmdFail(result, "Successfully added record of type SOA, " \
+        self.assertCmdFail(result, "Successfully added record of type SOA, "
                            "when this type should not be available.")
         self.assertTrue("type SOA is not supported" in err,
-                        "Invalid error message '%s' when attempting to " \
+                        "Invalid error message '%s' when attempting to "
                         "add record of type SOA." % err)
 
     def test_add_overlapping_different_type(self):
@@ -545,7 +546,7 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                                   self.zone, "testrecord",
                                                   dnstype1, record1,
                                                   self.creds_string)
-                self.assertCmdSuccess(result, out, err, "Failed to add record " \
+                self.assertCmdSuccess(result, out, err, "Failed to add record "
                                       "'%s' of type '%s'." % (record1, dnstype1))
 
                 result, out, err = self.runsubcmd("dns", "add",
@@ -553,8 +554,8 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                                   self.zone, "testrecord",
                                                   dnstype2, record2,
                                                   self.creds_string)
-                self.assertCmdSuccess(result, out, err, "Failed to add record " \
-                                      "'%s' of type '%s' when a record '%s' " \
+                self.assertCmdSuccess(result, out, err, "Failed to add record "
+                                      "'%s' of type '%s' when a record '%s' "
                                       "of type '%s' with the same name exists."
                                       % (record1, dnstype1, record2, dnstype2))
 
@@ -562,9 +563,9 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                                   os.environ["SERVER"],
                                                   self.zone, "testrecord",
                                                   dnstype1, self.creds_string)
-                self.assertCmdSuccess(result, out, err, "Failed to query for " \
-                                      "record '%s' of type '%s' when a new " \
-                                      "record '%s' of type '%s' with the same " \
+                self.assertCmdSuccess(result, out, err, "Failed to query for "
+                                      "record '%s' of type '%s' when a new "
+                                      "record '%s' of type '%s' with the same "
                                       "name was added."
                                       % (record1, dnstype1, record2, dnstype2))
 
@@ -572,9 +573,9 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                                   os.environ["SERVER"],
                                                   self.zone, "testrecord",
                                                   dnstype2, self.creds_string)
-                self.assertCmdSuccess(result, out, err, "Failed to query " \
-                                      "record '%s' of type '%s' which should " \
-                                      "have been added with the same name as " \
+                self.assertCmdSuccess(result, out, err, "Failed to query "
+                                      "record '%s' of type '%s' which should "
+                                      "have been added with the same name as "
                                       "record '%s' of type '%s'."
                                       % (record2, dnstype2, record1, dnstype1))
 
@@ -583,7 +584,7 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                                   self.zone, "testrecord",
                                                   dnstype1, record1,
                                                   self.creds_string)
-                self.assertCmdSuccess(result, out, err, "Failed to delete " \
+                self.assertCmdSuccess(result, out, err, "Failed to delete "
                                       "record '%s' of type '%s'."
                                       % (record1, dnstype1))
 
@@ -592,7 +593,7 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                                   self.zone, "testrecord",
                                                   dnstype2, record2,
                                                   self.creds_string)
-                self.assertCmdSuccess(result, out, err, "Failed to delete " \
+                self.assertCmdSuccess(result, out, err, "Failed to delete "
                                       "record '%s' of type '%s'."
                                       % (record2, dnstype2))
 
@@ -660,14 +661,171 @@ class DnsCmdTestCase(SambaToolCmdTest):
                                           "A", self.testip, self.creds_string)
         self.assertCmdFail(result)
 
+    def test_cleanup_record(self):
+        """
+        Test dns cleanup command is working fine.
+        """
+
+        # add a A record
+        self.runsubcmd("dns", "add", os.environ["SERVER"], self.zone,
+                       'testa', "A", self.testip, self.creds_string)
+
+        # the above A record points to this host
+        dnshostname = '{0}.{1}'.format('testa', self.zone.lower())
+
+        # add a CNAME record points to above host
+        self.runsubcmd("dns", "add", os.environ["SERVER"], self.zone,
+                       'testcname', "CNAME", dnshostname, self.creds_string)
+
+        # add a NS record
+        self.runsubcmd("dns", "add", os.environ["SERVER"], self.zone,
+                       'testns', "NS", dnshostname, self.creds_string)
+
+        # add a PTR record points to above host
+        self.runsubcmd("dns", "add", os.environ["SERVER"], self.zone,
+                       'testptr', "PTR", dnshostname, self.creds_string)
+
+        # add a SRV record points to above host
+        srv_record = "{0} 65530 65530 65530".format(dnshostname)
+        self.runsubcmd("dns", "add", os.environ["SERVER"], self.zone,
+                       'testsrv', "SRV", srv_record, self.creds_string)
+
+        # cleanup record for this dns host
+        self.runsubcmd("dns", "cleanup", os.environ["SERVER"],
+                       dnshostname, self.creds_string)
+
+        # all records should be marked as dNSTombstoned
+        for record_name in ['testa', 'testcname', 'testns', 'testptr', 'testsrv']:
+
+            records = self.samdb.search(
+                base="DC=DomainDnsZones,{0}".format(self.samdb.get_default_basedn()),
+                scope=ldb.SCOPE_SUBTREE,
+                expression="(&(objectClass=dnsNode)(name={0}))".format(record_name),
+                attrs=["dNSTombstoned"])
+
+            self.assertEqual(len(records), 1)
+            for record in records:
+                self.assertEqual(str(record['dNSTombstoned']), 'TRUE')
+
+    def test_cleanup_record_no_A_record(self):
+        """
+        Test dns cleanup command works with no A record.
+        """
+
+        # add a A record
+        self.runsubcmd("dns", "add", os.environ["SERVER"], self.zone,
+                       'notesta', "A", self.testip, self.creds_string)
+
+        # the above A record points to this host
+        dnshostname = '{0}.{1}'.format('testa', self.zone.lower())
+
+        # add a CNAME record points to above host
+        self.runsubcmd("dns", "add", os.environ["SERVER"], self.zone,
+                       'notestcname', "CNAME", dnshostname, self.creds_string)
+
+        # add a NS record
+        self.runsubcmd("dns", "add", os.environ["SERVER"], self.zone,
+                       'notestns', "NS", dnshostname, self.creds_string)
+
+        # add a PTR record points to above host
+        self.runsubcmd("dns", "add", os.environ["SERVER"], self.zone,
+                       'notestptr', "PTR", dnshostname, self.creds_string)
+
+        # add a SRV record points to above host
+        srv_record = "{0} 65530 65530 65530".format(dnshostname)
+        self.runsubcmd("dns", "add", os.environ["SERVER"], self.zone,
+                       'notestsrv', "SRV", srv_record, self.creds_string)
+
+        # Remove the initial A record (leading to hanging references)
+        self.runsubcmd("dns", "delete", os.environ["SERVER"], self.zone,
+                       'notesta', "A", self.testip, self.creds_string)
+
+        # cleanup record for this dns host
+        self.runsubcmd("dns", "cleanup", os.environ["SERVER"],
+                       dnshostname, self.creds_string)
+
+        # all records should be marked as dNSTombstoned
+        for record_name in ['notestcname', 'notestns', 'notestptr', 'notestsrv']:
+
+            records = self.samdb.search(
+                base="DC=DomainDnsZones,{0}".format(self.samdb.get_default_basedn()),
+                scope=ldb.SCOPE_SUBTREE,
+                expression="(&(objectClass=dnsNode)(name={0}))".format(record_name),
+                attrs=["dNSTombstoned"])
+
+            self.assertEqual(len(records), 1)
+            for record in records:
+                self.assertEqual(str(record['dNSTombstoned']), 'TRUE')
+
+    def test_cleanup_multi_srv_record(self):
+        """
+        Test dns cleanup command for multi-valued SRV record.
+
+        Steps:
+        - Add 2 A records host1 and host2
+        - Add a SRV record srv1 and points to both host1 and host2
+        - Run cleanup command for host1
+        - Check records for srv1, data for host1 should be gone and host2 is kept.
+        """
+
+        hosts = ['host1', 'host2']  # A record names
+        srv_name = 'srv1'
+
+        # add A records
+        for host in hosts:
+            self.runsubcmd("dns", "add", os.environ["SERVER"], self.zone,
+                           host, "A", self.testip, self.creds_string)
+
+            # the above A record points to this host
+            dnshostname = '{0}.{1}'.format(host, self.zone.lower())
+
+            # add a SRV record points to above host
+            srv_record = "{0} 65530 65530 65530".format(dnshostname)
+            self.runsubcmd("dns", "add", os.environ["SERVER"], self.zone,
+                           srv_name, "SRV", srv_record, self.creds_string)
+
+        records = self.samdb.search(
+            base="DC=DomainDnsZones,{0}".format(self.samdb.get_default_basedn()),
+            scope=ldb.SCOPE_SUBTREE,
+            expression="(&(objectClass=dnsNode)(name={0}))".format(srv_name),
+            attrs=['dnsRecord'])
+        # should have 2 records here
+        self.assertEqual(len(records[0]['dnsRecord']), 2)
+
+        # cleanup record for dns host1
+        dnshostname1 = 'host1.{0}'.format(self.zone.lower())
+        self.runsubcmd("dns", "cleanup", os.environ["SERVER"],
+                       dnshostname1, self.creds_string)
+
+        records = self.samdb.search(
+            base="DC=DomainDnsZones,{0}".format(self.samdb.get_default_basedn()),
+            scope=ldb.SCOPE_SUBTREE,
+            expression="(&(objectClass=dnsNode)(name={0}))".format(srv_name),
+            attrs=['dnsRecord', 'dNSTombstoned'])
+
+        # dnsRecord for host1 should be deleted
+        self.assertEqual(len(records[0]['dnsRecord']), 1)
+
+        # unpack data
+        dns_record_bin = records[0]['dnsRecord'][0]
+        dns_record_obj = ndr_unpack(dnsp.DnssrvRpcRecord, dns_record_bin)
+
+        # dnsRecord for host2 is still there and is the only one
+        dnshostname2 = 'host2.{0}'.format(self.zone.lower())
+        self.assertEqual(dns_record_obj.data.nameTarget, dnshostname2)
+
+        # assert that the record isn't spuriously tombstoned
+        self.assertTrue('dNSTombstoned' not in records[0] or
+                        str(records[0]['dNSTombstoned']) == 'FALSE')
+
     def test_dns_wildcards(self):
         """
         Ensure that DNS wild card entries can be added deleted and queried
         """
         num_failures = 0
         failure_msgs = []
-        records = [("*.",       "MISS",         "A", "1.1.1.1"),
-                   ("*.SAMDOM", "MISS.SAMDOM",  "A", "1.1.1.2")]
+        records = [("*.", "MISS", "A", "1.1.1.1"),
+                   ("*.SAMDOM", "MISS.SAMDOM", "A", "1.1.1.2")]
         for (name, miss, dnstype, record) in records:
             try:
                 result, out, err = self.runsubcmd("dns", "add",
@@ -726,3 +884,29 @@ class DnsCmdTestCase(SambaToolCmdTest):
                 print(msg)
             self.fail("Failed to accept valid commands. %d total failures."
                       "Errors above." % num_failures)
+
+    def test_serverinfo(self):
+        for v in ['w2k', 'dotnet', 'longhorn']:
+            result, out, err = self.runsubcmd("dns",
+                                              "serverinfo",
+                                              "--client-version", v,
+                                              os.environ["SERVER"],
+                                              self.creds_string)
+            self.assertCmdSuccess(result,
+                                  out,
+                                  err,
+                                  "Failed to print serverinfo with "
+                                  "client version %s" % v)
+            self.assertTrue(out != '')
+
+    def test_zoneinfo(self):
+        result, out, err = self.runsubcmd("dns",
+                                          "zoneinfo",
+                                          os.environ["SERVER"],
+                                          self.zone,
+                                          self.creds_string)
+        self.assertCmdSuccess(result,
+                              out,
+                              err,
+                              "Failed to print zoneinfo")
+        self.assertTrue(out != '')
